searchNodes=[{"doc":"A use-able diagnostic plugin module A diagnostic result examines a diagnosable data structure and emits a list of Lexical.Plugin.V1.Diagnostic.Result structs. Diagnostic plugins are called in two places. When a file is saved, they're called with the Lexical.Project struct and are expected to perform diagnostics across the whole project. On receipt of a change to a Lexical.Document , they're called with the changed document and are expected to analyze that document and emit any diagnostics they find. Both calls to diagnose operate on tight deadlines, though when diagnose is called with a Lexical.Project , the deadline is much longer than it is when diagnose is called with a Lexical.Document . This is because analyzing a project should take a lot longer than analyzing a single document. Currently, Lexical sets a deadline of around a second on project-level diagnostics and a tens-of-milliseconds deadline on document plugins. Errors and Timeouts Plugins are very sensitive to errors and are disabled by lexical if they cause too many. At present, a disabled plugin can only be re-enabled by restarting lexical, so ensure that your plugin doesn't crash, and if you don't have diagnostics, return {:ok, []} rather than some error response. Plugin lifecycle When a plugin is started, Lexical calls the init/0 function, which can perform setup actions, like starting the application associated with the plugin. Implementing this function is optional. From then on, the plugin will be resident and run in tasks whenever files are changed or saved. A simple do-nothing plugin defmodule DoesNothing do use Lexical.Plugin.V1.Diagnostic def diagnose ( % Lexical.Document { } = doc ) do { :ok , [ ] } end def diagnose ( % Lexical.Project { } ) do { :ok , [ ] } end end Check out the README for a plugin that does something more.","ref":"Lexical.Plugin.V1.Diagnostic.html","title":"Lexical.Plugin.V1.Diagnostic","type":"behaviour"},{"doc":"","ref":"Lexical.Plugin.V1.Diagnostic.html#c:diagnose/1","title":"Lexical.Plugin.V1.Diagnostic.diagnose/1","type":"callback"},{"doc":"","ref":"Lexical.Plugin.V1.Diagnostic.html#t:diagnosable/0","title":"Lexical.Plugin.V1.Diagnostic.diagnosable/0","type":"type"},{"doc":"","ref":"Lexical.Plugin.V1.Diagnostic.html#t:diagnostics_reply/0","title":"Lexical.Plugin.V1.Diagnostic.diagnostics_reply/0","type":"type"},{"doc":"","ref":"Lexical.Plugin.V1.Diagnostic.html#t:results/0","title":"Lexical.Plugin.V1.Diagnostic.results/0","type":"type"},{"doc":"","ref":"Lexical.Plugin.V1.Diagnostic.html#t:state/0","title":"Lexical.Plugin.V1.Diagnostic.state/0","type":"type"},{"doc":"The result of a diagnostic run A diagnostic plugin emits a list of Result structs that inform the user about issues the plugin has found. The results contain the following keys: uri - The URI of the document where the error occurs. Lexical.Document structs contain a uri field which can be used to fill out this field. If you have a filesystem path, the function Lexical.Document.Path.to_uri/1 can be used to transform a path to a URI. message - The diagnostic message displayed to the user details - Further details about the message position - Where the message occurred (see the Positions section for details) severity - How important the issue is. Can be one of (from least severe to most severe) :hint , :information , :warning , :error source - The name of the plugin that produced this as a human-readable string. Positions Diagnostics need to inform the language client where the error occurs. Positions are the mechanism they use to do so. Positions take several forms, which are: line number - If the position is a one-based line number, the diagnostic will refer to the entire flagged line {line_number, column} - If the position is a two-tuple of a one-based line number and a one-based column, then the diagnostic will begin on the line indicated, and start at the column indicated. The diagnostic will run to the end of the line. {start_line, start_column, end_line, end_column} - If the position is a four-tuple of of one-based line and column numbers, the diagnostic will start on start_line at start_column and run until end_line at end_column . This is the most detailed form of describing a position, and should be preferred to the others, as it will produce the most accurate highlighting of the diagnostic. Document.Range.t - Equivalent to the {start_line, start_column, end_line, end_column}, but saves a conversion step Document.Position.t - Equivalent to {line_number, column} , but saves a conversion step.","ref":"Lexical.Plugin.V1.Diagnostic.Result.html","title":"Lexical.Plugin.V1.Diagnostic.Result","type":"module"},{"doc":"Creates a new diagnostic result.","ref":"Lexical.Plugin.V1.Diagnostic.Result.html#new/6","title":"Lexical.Plugin.V1.Diagnostic.Result.new/6","type":"function"},{"doc":"A path or a uri.","ref":"Lexical.Plugin.V1.Diagnostic.Result.html#t:path_or_uri/0","title":"Lexical.Plugin.V1.Diagnostic.Result.path_or_uri/0","type":"type"},{"doc":"Where the error occurs in the document.","ref":"Lexical.Plugin.V1.Diagnostic.Result.html#t:position/0","title":"Lexical.Plugin.V1.Diagnostic.Result.position/0","type":"type"},{"doc":"The severity of the diagnostic.","ref":"Lexical.Plugin.V1.Diagnostic.Result.html#t:severity/0","title":"Lexical.Plugin.V1.Diagnostic.Result.severity/0","type":"type"},{"doc":"A result emitted by a diagnostic plugin. These results are displayed in the editor to the user.","ref":"Lexical.Plugin.V1.Diagnostic.Result.html#t:t/0","title":"Lexical.Plugin.V1.Diagnostic.Result.t/0","type":"type"},{"doc":"Extend lexical's functionality","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"Plugins are used to extend lexical's functionality and provide opt-in diagnostics, completion, and code intelligence features without having to build inside of Lexical itself.","ref":"readme.html#overview","title":"Overview - Overview","type":"extras"},{"doc":"The goal of the plugin project is to have different types of plugins that affect lexical in different ways. Presently, only diagnostic plugins are supported. A diagnostic examines a mix project or a document and emits Lexical.Plugin.V1.Diagnostic.Result structs to direct the user's attention to any issues it finds. Diagnostic plugins can be used to integrate code linters like credo or to enforce project-specific rules and coding practices. Creating a diagnostic plugin Create a new mix project with mix new my_plugin , and edit the mix.exs file to include the :lexical_plugin app. def deps do [ { :lexical_plugin , &quot;~&gt; 0.1.0&quot; } ] end You will also need to add an application env key in your mix.exs to tell lexical that this is a plugin application. When an application is marked like this, its modules are searched for any that implement plugin behaviours. Add the application key like this: def application do [ extra_applications : [ :logger ] , env : [ lexical_plugin : true ] ] Now we implement the plugin module. In our module, we're going to emit an error on the first line of every file we encounter with a message. defmodule NoisyPlugin do alias Lexical.Document alias Lexical.Plugin.V1.Diagnostic alias Lexical.Project use Diagnostic , name : :noisy_example_plugin def diagnose ( % Document { } = doc ) do results = if Document . size ( doc ) &gt;= 1 do [ build_result ( doc . path ) ] else [ ] end { :ok , results } end def diagnose ( % Project { } = project ) do root_path = Project . root_path ( project ) glob = if umbrella? ( root_path ) do &quot; \#{ root_path } /apps/**/*.ex&quot; else &quot; \#{ root_path } /lib/**/*.ex&quot; end results = glob |&gt; Path . wildcard ( ) |&gt; Enum . map ( &amp; build_result / 1 ) { :ok , results } end defp build_result ( document_path ) do Diagnostic.Result . new ( document_path , 1 , &quot;Do you want to start like this?&quot; , :information , &quot;Noisy Plugin&quot; ) end defp umbrella? ( root_path ) do root_path |&gt; Path . join ( &quot;apps&quot; ) |&gt; File . dir? ( ) end end ...and that's it. Now, you can install that plugin in your project by adding it to the project's mix.exs , and when lexical starts, it will detect the plugin and every file in your project will have a little noisy error at the top.","ref":"readme.html#plugin-types","title":"Overview - Plugin Types","type":"extras"},{"doc":"Install this dependency as an optional dep in your project def deps do [ { :lexical_plugin , &quot;~&gt; 0.1.0&quot; , optional : true } ] end Documentation can be found at https://hexdocs.pm/lexical_plugin .","ref":"readme.html#installation","title":"Overview - Installation","type":"extras"}]